# Calculation of fuel required for flight based on input params

module Fuelator
  # Calculate fuel for a flight
  #
  # Example:
  #   >> Fuelator.Calc.new(28801, [[:launch, 9.807], [:land, 1.62], [:launch, 1.62], [:land, 9.807]]).run
  #   => 51898
  class Calc
    attr_reader :parameters

    FLIGHT_CONSTANTS = {
      launch: {
        theta: 0.042,
        bias:  33,
      },
      land:   {
        theta: 0.033,
        bias:  42,
      },
    }

    def self.run(mass, pairs)
      new(mass, pairs).run
    end

    def initialize(mass, pairs)
      @parameters = Parameters.new(mass, pairs)
    end

    def run
      parameters.validate!
      full_mass = parameters.mass

      parameters.reversed.map do |val|
        full_mass += recursive_calculate(full_mass, val)
      end.last - parameters.mass
    end

    private

    def recursive_calculate(mass, pair)
      constants = FLIGHT_CONSTANTS[pair.directive]
      fuel_mass = (mass * pair.gravity * constants[:theta] - constants[:bias]).to_i # rounded down

      return 0 if fuel_mass <= 0
      result = recursive_calculate(fuel_mass, pair)
      fuel_mass + result
    end
  end
end