# For flight parameters validation purposes

module Fuelator
  class Parameters
    include Validator

    class Error < StandardError; end

    attr_reader :mass, :pairs, :order

    def initialize(mass, pairs)
      @mass  = mass
      @pairs = pairs.map { |pair| Pair.new(pair).validate! }
      @order = define_order
    end

    def reversed
      pairs.reverse
    end

    def valid_mass?
      mass.is_a?(Numeric)
    end

    def valid_pairs?
      !pairs.empty?
    end

    def valid_order?
      !order
    end

    private

    # you can only calculate if directives passed in right order - :launch->:land
    def define_order
      dir = nil
      pairs.map do |pair|
        return true if dir == pair.directive

        dir = pair.directive
      end
      false
    end
  end
end
